#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>

#define DREAPTA OCR2
#define STANGA OCR0
#define MAX_SPEED 182

void init_PWM()
{
	// 1. initializare pin
	DDRB |= (1<<PB3);
	// 2. Fast PWM, 1024 prescaler, non-inverting mode
	TCCR0 |= (1<<WGM01)|(1<<WGM00);
	TCCR0 |= (1<<CS02)|(1<<CS00);
	TCCR0 &= ~(1<<CS01);
	//non-inverting mode
	TCCR0 |= (1<<COM01);
	TCCR0 &= ~(1<<COM00);
	
	// 1. initializare pin
	DDRD |= (1<<PD7);
	// 2. Fast PWM, 1024 prescaler, non-inverting mode
	TCCR2 |= (1<<WGM21)|(1<<WGM20);
	TCCR2 |= (1<<CS20)|(1<<CS21)|(1<<CS22);
	//non-inverting mode
	TCCR2 |= (1<<COM21);
	TCCR2 &= ~(1<<COM20);
}

int main(void)
{
	sei();
	
	init_PWM();
	
	// Senzori
	// Se activeaza rezistentele de pull-up
	PORTC = 0xff;
	DDRC = 0x00;	
	
	// Control Driver Motoare
	DDRD |= (1<<PD1)|(1<<PD2)|(1<<PD3)|(1<<PD4);
	
	int stanga, dreapta;
	
	while(1)
    {
		// Se face case dupa PINC ( valorile zenzorilor digitali )
		//si se decide turatia pe motoare
		switch (PINC)
		{
			case 0b10000000:
				stanga = - MAX_SPEED / 2;
				dreapta = MAX_SPEED;
				break;
			case 0b11000000:
				stanga = 0;
				dreapta = MAX_SPEED;
				break;
			case 0b01000000:
				stanga = 17 / 100 * MAX_SPEED;
				dreapta = MAX_SPEED;
				break;
			case 0b01100000:
				stanga = 35 / 100 * MAX_SPEED;
				dreapta = MAX_SPEED;
				break;
			case 0b00100000:
				stanga = 50 / 100 * MAX_SPEED;
				dreapta = MAX_SPEED;
				break;
			case 0b00110000:
				stanga = 75 / 100 * MAX_SPEED;
				dreapta = MAX_SPEED;
				break;
			case 0b00010000:
				stanga = 90 / 100 * MAX_SPEED;
				dreapta = MAX_SPEED;
				break;
			case 0b00011000:
				stanga = dreapta = MAX_SPEED;
				break;
			case 0b00001000:
				dreapta = 90 / 100 * MAX_SPEED;
				stanga = MAX_SPEED;
				break;
			case 0b00001100:
				dreapta = 75 / 100 * MAX_SPEED;
				stanga = MAX_SPEED;
				break;
			case 0b00000100:
				dreapta = 50 / 100 * MAX_SPEED;
				stanga = MAX_SPEED;
				break;
			case 0b00000110:
				dreapta = 35 / 100 * MAX_SPEED;
				stanga = MAX_SPEED;
				break;
			case 0b00000010:
				dreapta = 17 / 100 * MAX_SPEED;
				stanga = MAX_SPEED;
				break;
			case 0b00000011:
				dreapta = 0;
				stanga = MAX_SPEED;
				break;
			case 0b00000001:
				dreapta = - MAX_SPEED / 2;
				stanga = MAX_SPEED;
				break;
			case 0b00000000:
				if (stanga < 0) {
					stanga = - MAX_SPEED / 2;
					dreapta = MAX_SPEED;
				} else {
					dreapta = - MAX_SPEED / 2;
					stanga = MAX_SPEED;
				}
				break;
		}
		// In functie de valoare, pozitiva sau negativa a lui
		//dreapta, se schimba sensul de mers al motoarelor
		if (dreapta < 0){ 
			PORTD |= (1<<PD5);
			PORTD &= ~(1<<PD4);
		}
		else {
			PORTD &= ~(1<<PD5);
			PORTD |= (1<<PD4);
		}
		
		// In functie de valoare, pozitiva sau negativa a lui
		//stanga, se schimba sensul de mers al motoarelor
		if (stanga > 0){
			PORTD |= (1<<PD2);
			PORTD &= ~(1<<PD3);
		}
		else {
			PORTD &= ~(1<<PD2);
			PORTD |= (1<<PD3);
		}
		
		// Se seteaza turatia de mers a motoarelor in valoare absoluta
		STANGA = abs(stanga);
		DREAPTA = abs(dreapta);
		_delay_ms(10);
    }
}